--[[
/*******************************************************************************\
*                                                                               *
* All-In-One Generic Script Pack                                                *
*                                                                               *
*********************************************************************************
* Licensed under the MIT or X11 License                                         *
*                                                                               *
* Copyright (c) 2014 Eisbearg                                              *
*                                                                               *
* Permission is hereby granted, free of charge, to any person obtaining a copy  *
* of this software and associated documentation files (the "Software"), to deal *
* in the Software without restriction, including without limitation the rights  *
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell     *
* copies of the Software, and to permit persons to whom the Software is         *
* furnished to do so, subject to the following conditions:                      *
*                                                                               *
* The above copyright notice and this permission notice shall be included in    *
* all copies or substantial portions of the Software.                           *
*                                                                               *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR    *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,      *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE   *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER        *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, *
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN     *
* THE SOFTWARE.                                                                 *
\*******************************************************************************/
]]--
-------------------------------------------------------------------------------------------------------------------------------
--
-- Utility Script that shows a simple compass in the upper left corner
--
-- @author	Eisbearg
-- @date	01.11.2014
--
-------------------------------------------------------------------------------------------------------------------------------

Compass = {};


Compass.position = {x=0.002, y=0.984};


Compass.drawCompass = function(self)
	if self:getIsActive() and self.isEntered then
		local direction = Compass:getDirection(self);
		-- always be on the safe side of life -> set text-styling
		setTextAlignment(RenderText.ALIGN_LEFT);
		setTextBold(false);
		setTextColor(1,0.1,0,1);
		renderText(Compass.position.x, Compass.position.y, 0.016, string.format("Compass: %6.2f°", direction));
		-- always be on the safe side of life -> reset text-styling
		setTextAlignment(RenderText.ALIGN_LEFT);
		setTextBold(false);
		setTextColor(1,1,1,1);
	end;
end;


function Compass:getDirection(steerableSelf)
	local x,y,z = localDirectionToWorld(steerableSelf.rootNode, 0, 0, 1);
	local length = Utils.vector2Length(x,z);

	if (length ~= 0.0) then
		local dX = x / length;
		local dZ = z / length;
		local direction = math.deg(math.atan2(dZ, dX)) + 90.0;
		while (direction > 359.999999) do
			direction = direction - 360.0;
		end;
		while (direction < 0.0) do
			direction = direction + 360.0;
		end;

		return direction;
	end;  
	return 0;
end;


-- adding extra functionality to existing functions, by using Utils.prependedFunction(), Utils.appendedFunction() or Utils.overwrittenFunction()
Steerable.draw = Utils.appendedFunction(Steerable.draw, Compass.drawCompass);


DebugExtension:printScriptSplash(g_currentModDirectory, "Simple Compass in the upper left corner");
