--[[
/*******************************************************************************\
*                                                                               *
* All-In-One Generic Script Pack                                                *
*                                                                               *
*********************************************************************************
* Licensed under the MIT or X11 License                                         *
*                                                                               *
* Copyright (c) 2014 Eisbearg                                              *
*                                                                               *
* Permission is hereby granted, free of charge, to any person obtaining a copy  *
* of this software and associated documentation files (the "Software"), to deal *
* in the Software without restriction, including without limitation the rights  *
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell     *
* copies of the Software, and to permit persons to whom the Software is         *
* furnished to do so, subject to the following conditions:                      *
*                                                                               *
* The above copyright notice and this permission notice shall be included in    *
* all copies or substantial portions of the Software.                           *
*                                                                               *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR    *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,      *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE   *
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER        *
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, *
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN     *
* THE SOFTWARE.                                                                 *
\*******************************************************************************/
]]--
-------------------------------------------------------------------------------------------------------------------------------
--
-- Utility Script to debug other scripts
--
-- @author	Eisbearg
-- @date	01.11.2014
--
-------------------------------------------------------------------------------------------------------------------------------




-------------------------------------------------------------------------------------------------------------------------------
--
-- global definitions
--
-------------------------------------------------------------------------------------------------------------------------------

DebugExtension = {};


-- you shall change the level to your needs (Debug prints a lot once, FunctionTrace will print a lot all the time)
DebugExtension.DebugLevel = 3; -- Nothing = 0, Errors = 1, Warnings = 2, Infos = 3, Debug = 4, FunctionTrace = 5, Debug DebugExtension = 6




-------------------------------------------------------------------------------------------------------------------------------
-- public methods that can be called from other scripts
-------------------------------------------------------------------------------------------------------------------------------
--
-- some output generators, that may be interesting
--
-------------------------------------------------------------------------------------------------------------------------------


function DebugExtension:printScriptSplash(baseDirectory, annotation)
	local splash = "";
	splash = splash.."##############################################################################################################################\n";
	splash = splash.."## Script activating\n";
	splash = splash.."## ScriptFile: "..DebugExtension:getPredecessorFileName().."\n";
	splash = splash.."## Version:    "..DebugExtension:getVersion(DebugExtension:unslashModDirectory(baseDirectory)).."\n";
	if annotation ~= nil then
		splash = splash.."## Annotation: "..annotation.."\n";
	end;
	splash = splash.."##############################################################################################################################";
	print(splash);
end;


function DebugExtension:printVehicleTypes()
	local splash = "";
	splash = splash.."##############################################################################################################################\n";
	splash = splash.."## VehicleTypes\n";
	splash = splash..DebugExtension:tableToString(VehicleTypeUtil.vehicleTypes, 1);
	splash = splash.."\n";
	splash = splash.."##############################################################################################################################";
	print(splash);
end;


function DebugExtension:printFruitTypes()
	local splash = "";
	splash = splash.."##############################################################################################################################\n";
	splash = splash.."## FruitTypes\n";
	splash = splash..DebugExtension:tableToString(FruitUtil.fruitTypes, 1);
	splash = splash.."\n";
	splash = splash.."##############################################################################################################################";
	print(splash);
end;


function DebugExtension:printFillables()
	local splash = "";
	splash = splash.."##############################################################################################################################\n";
	splash = splash.."## FruitTypes\n";
	splash = splash..DebugExtension:tableToString(Fillable, 1);
	splash = splash.."\n";
	splash = splash.."##############################################################################################################################";
	print(splash);
end;


-------------------------------------------------------------------------------------------------------------------------------
-- public methods that can be called from other scripts
-------------------------------------------------------------------------------------------------------------------------------
-- 
-- Logging facility
--
-------------------------------------------------------------------------------------------------------------------------------

local sm_MessagePrefix = "    DebugExtension(%s) %s(%s).%s():> "; -- this is the prefix of every debug output



function DebugExtension:logError(annotation)
	if DebugExtension.DebugLevel < 1 then
		return;
	end;

	local msg = string.format(sm_MessagePrefix, "ERROR", DebugExtension:getPredecessorFileName(), DebugExtension:getPredecessorLineName(), DebugExtension:getPredecessorFunctionName());
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;



function DebugExtension:logWarning(annotation)
	if DebugExtension.DebugLevel < 2 then
		return;
	end;

	local msg = string.format(sm_MessagePrefix, "WARNG", DebugExtension:getPredecessorFileName(), DebugExtension:getPredecessorLineName(), DebugExtension:getPredecessorFunctionName());
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;



function DebugExtension:logInfo(annotation)
	if DebugExtension.DebugLevel < 3 then
		return;
	end;

	local msg = string.format(sm_MessagePrefix, "INFOS", DebugExtension:getPredecessorFileName(), DebugExtension:getPredecessorLineName(), DebugExtension:getPredecessorFunctionName());
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;



function DebugExtension:logDebug(annotation)
	if DebugExtension.DebugLevel < 4 then
		return;
	end;

	local msg = string.format(sm_MessagePrefix, "DEBUG", DebugExtension:getPredecessorFileName(), DebugExtension:getPredecessorLineName(), DebugExtension:getPredecessorFunctionName());
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;



function DebugExtension:logFunctionEntry(annotation)
	if DebugExtension.DebugLevel < 5 then
		return;
	end;

	local msg = string.format(sm_MessagePrefix, "ENTER", DebugExtension:getPredecessorFileName(), DebugExtension:getPredecessorLineName(), DebugExtension:getPredecessorFunctionName());
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;



function DebugExtension:logFunctionExit(annotation)
	if DebugExtension.DebugLevel < 5 then
		return;
	end;

	local msg = string.format(sm_MessagePrefix, "EXISTS", DebugExtension:getPredecessorFileName(), DebugExtension:getPredecessorLineName(), DebugExtension:getPredecessorFunctionName());
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;


-------------------------------------------------------------------------------------------------------------------------------
-- public methods that can be called from other scripts
-------------------------------------------------------------------------------------------------------------------------------
-- 
-- Listener interface
--
-------------------------------------------------------------------------------------------------------------------------------


function DebugExtension:loadMap(name)
	if DebugExtension.DebugLevel < 3 then
		return;
	end;
	DebugExtension:logInfo("loadMap("..tostring(name)..")");

	if DebugExtension.DebugLevel < 4 then
		return;
	end;
	DebugExtension:printVehicleTypes();
	DebugExtension:printFruitTypes();
	DebugExtension:printFillables();
end;


function DebugExtension:deleteMap()
end;


function DebugExtension:delete()
end;


function DebugExtension:mouseEvent(posX, posY, isDown, isUp, button)
end;


function DebugExtension:keyEvent(unicode, sym, modifier, isDown)
end;


function DebugExtension:update(dt)
end;


function DebugExtension:draw()
end;


addModEventListener(DebugExtension);



-------------------------------------------------------------------------------------------------------------------------------
--
-- private utility methods
--
-------------------------------------------------------------------------------------------------------------------------------

local sm_DebugMsgTmplt = "    DebugExtension(%s) DebugExtension.%s():> ";



function DebugExtension:traceDebExt(method, annotation)
	if DebugExtension.DebugLevel < 6 then
		return;
	end;

	local msg = string.format(sm_DebugMsgTmplt, "DEBEX", method);
	if annotation ~= nil then
		 msg =  msg..annotation;
	end;
	
	print(msg);
end;



function DebugExtension:unslashModDirectory(baseDirectory)
	if baseDirectory == nil then
		return baseDirectory;
	end;
	
	return baseDirectory:gsub("%//", "/");
end;



function DebugExtension:getVersion(baseDirectory)
	if baseDirectory == nil then
		DebugExtension:traceDebExt("getVersion", "baseDirectory == nil : please provide g_currentModDirectory or self.baseDirectory");
		return "0.0";
	end;
	
	local modDescFile = Utils.getFilename("modDesc.xml", DebugExtension:unslashModDirectory(baseDirectory)); 
	if modDescFile == nil then
		DebugExtension:traceDebExt("getVersion", "modDesc.xml not found");
		return "0.0";
	end;

	modDescFile = loadXMLFile("modDescFile", modDescFile);
	if modDescFile == nil then
		DebugExtension:traceDebExt("getVersion", "cannot load modDesc.xml");
		return "0.0";
	end;

	local version = getXMLString(modDescFile, "modDesc.version");
	if version == nil then
		DebugExtension:traceDebExt("getVersion", "cannot find version element");
		return "0.0";
	end;
	
	return version;
end;



function DebugExtension:getPredecessorFileName()
	local calleeFileName = debug.getinfo(3).short_src;
	if calleeFileName == nil then
		DebugExtension:traceDebExt("getPredecessorFileName", "calleeFileName not found");
		return "<UnknownFile>";
	end;
	
	return DebugExtension:unslashModDirectory(calleeFileName);
end;



function DebugExtension:getPredecessorLineName()
	local calleeLineName = debug.getinfo(3).currentline;
	if calleeLineName == nil then
		DebugExtension:traceDebExt("getPredecessorLineName", "calleeLineName not found");
		return "<UnknownLine>";
	end;
	
	return calleeLineName;
end;



function DebugExtension:getPredecessorFunctionName()
	local calleeFunctionName = debug.getinfo(3).name;
	if calleeFunctionName == nil then
		DebugExtension:traceDebExt("getPredecessorFunctionName", "calleeFunctionName not found");
		return "<UnknownFunction>";
	end;
	
	return calleeFunctionName;
end;



function DebugExtension:tableToString(tt, indent, done)
	done = done or {}
	indent = indent or 0
	if type(tt) == "table" then
		local sb = {}
		for key, value in pairs (tt) do
			table.insert(sb, string.rep (" ", indent)) -- indent it
			if type (value) == "table" and not done [value] then
				done [value] = true
				table.insert(sb, "{\n");
				table.insert(sb, DebugExtension:tableToString (value, indent + 2, done))
				table.insert(sb, string.rep (" ", indent)) -- indent it
				table.insert(sb, "}\n");
			elseif "number" == type(key) then
				table.insert(sb, string.format("\"%s\"\n", tostring(value)))
			else
				table.insert(sb, string.format("%s = \"%s\"\n", tostring(key), tostring(value)))
			end
		end
		return table.concat(sb)
	else
		return tt .. "\n"
	end
end
